/* This file was generated by syncqt. */
#ifndef QT_QTLOTTIEVECTORIMAGEGENERATOR_VERSION_H
#define QT_QTLOTTIEVECTORIMAGEGENERATOR_VERSION_H

#define QTLOTTIEVECTORIMAGEGENERATOR_VERSION_STR "6.11.1"

#define QTLOTTIEVECTORIMAGEGENERATOR_VERSION 0x0601101

#endif // QT_QTLOTTIEVECTORIMAGEGENERATOR_VERSION_H
